@@ -18,6 +18,7 @@ pub enum AkitaError {
     UrlParseError(String),
     RedundantField(String),
     UnknownDatabase(String),
+    UnsupportedOperation(String),
     Unknown,
 }
 
@@ -28,6 +29,7 @@ impl fmt::Display for AkitaError {
             AkitaError::InvalidSQL(ref err) => err.fmt(f),
             AkitaError::InvalidField(ref err) => err.fmt(f),
             AkitaError::ExcuteSqlError(ref err, ref sql) => write!(f, "SQL Excute Error: {}, SQL: {}", err, sql),
+            AkitaError::UnsupportedOperation(ref err) => write!(f, "Unsupported operation: {}", err),
             AkitaError::UnknownDatabase(ref schema) => write!(f, "Unknown Database URL :{} (Just Support MySQL)", schema),
             AkitaError::MissingIdent(ref err) => err.fmt(f),
             AkitaError::UrlParseError(ref err) => err.fmt(f),
@@ -51,6 +53,7 @@ impl std::error::Error for AkitaError {
             AkitaError::InvalidSQL(ref err) => err,
             AkitaError::ExcuteSqlError(ref err, ref _sql) => err,
             AkitaError::InvalidField(ref err) => err,
+            AkitaError::UnsupportedOperation(ref err) => err,
             AkitaError::UrlParseError(ref err) => err,
             AkitaError::MissingIdent(ref err) => err,
             AkitaError::DataError(ref err) => err,
