@@ -1,12 +1,12 @@
-use crate::{self as akita, AkitaError, IPage, UpdateWrapper, Wrapper, database::{Database, DatabasePlatform}, information::{DatabaseName, FieldName, FieldType, GetFields, GetTableName, TableDef, TableName}, mapper::AkitaMapper, value::{ToValue, Value}};
+use crate::{self as akita, AkitaError, IPage, Params, UpdateWrapper, Wrapper, database::{Database, DatabasePlatform}, information::{DatabaseName, FieldName, FieldType, GetFields, GetTableName, TableDef, TableName}, mapper::AkitaMapper, value::{ToValue, Value}};
 use crate::data::{FromAkita, Rows, AkitaData, ToAkita};
 /// an interface executing sql statement and getting the results as generic Akita values
 /// without any further conversion.
 #[allow(unused)]
-pub struct AkitaManager(pub DatabasePlatform);
+pub struct AkitaManager(DatabasePlatform);
 
 #[allow(unused)]
-pub struct AkitaEntityManager(pub DatabasePlatform);
+pub struct AkitaEntityManager(DatabasePlatform);
 
 pub struct AkitaTransaction<'a> {
     pub(crate) conn: &'a mut AkitaEntityManager,
@@ -27,7 +27,6 @@ impl AkitaTransaction <'_> {
         self.rolled_back = true;
         Ok(())
     }
-
 }
 
 impl<'a> Drop for AkitaTransaction<'a> {
@@ -102,7 +101,6 @@ impl AkitaMapper for AkitaTransaction <'_> {
             self.conn.remove_by_id::<T, I>(id)
         
     }
-    
 
     /// Update the records by wrapper.
     fn update<T>(&mut self, entity: &T, wrapper: &mut UpdateWrapper) -> Result<(), AkitaError> 
@@ -136,10 +134,10 @@ impl AkitaMapper for AkitaTransaction <'_> {
     }
 
     #[allow(clippy::redundant_closure)]
-    fn execute_result<'a, R>(
+    fn execute_result<'a, R, S: Into<String>, P: Into<Params>>(
         &mut self,
-        sql: &str,
-        params: &[&'a dyn ToValue],
+        sql: S,
+        params: P,
     ) -> Result<Vec<R>, AkitaError>
     where
         R: FromAkita,
@@ -147,19 +145,19 @@ impl AkitaMapper for AkitaTransaction <'_> {
         self.conn.execute_result(sql, params)
     }
 
-    fn execute_drop<'a, S: Into<String>>(
+    fn execute_drop<'a, S: Into<String>, P: Into<Params>>(
         &mut self,
         sql: S,
-        params: &[&'a dyn ToValue],
+        params: P,
     ) -> Result<(), AkitaError>
     {
         self.conn.execute_drop(sql, params)
     }
 
-    fn execute_first<'a, R, S: Into<String>>(
+    fn execute_first<'a, R, S: Into<String>, P: Into<Params>>(
         &mut self,
         sql: S,
-        params: &[&'a dyn ToValue],
+        params: P,
     ) -> Result<R, AkitaError>
     where
         R: FromAkita,
@@ -167,10 +165,10 @@ impl AkitaMapper for AkitaTransaction <'_> {
         self.conn.execute_first(sql, params)
     }
 
-    fn execute_result_opt<'a, R, S: Into<String>>(
+    fn execute_result_opt<'a, R, S: Into<String>, P: Into<Params>>(
         &mut self,
         sql: S,
-        params: &[&'a dyn ToValue],
+        params: P,
     ) -> Result<Option<R>, AkitaError>
     where
         R: FromAkita,
@@ -182,6 +180,11 @@ impl AkitaMapper for AkitaTransaction <'_> {
 
 #[allow(unused)]
 impl AkitaManager {
+
+    pub fn new(db: DatabasePlatform) -> Self {
+        AkitaManager(db)
+    }
+
     pub fn start_transaction(&mut self) -> Result<(), AkitaError> {
         self.0.start_transaction()
     }
@@ -194,29 +197,29 @@ impl AkitaManager {
         self.0.rollback_transaction()
     }
 
-    pub fn execute_result(
+    pub fn execute_result<S: Into<String>, P: Into<Params>>(
         &mut self,
-        sql: &str,
-        params: &[&Value],
+        sql: S,
+        params: P,
     ) -> Result<Rows, AkitaError> {
-        let rows = self.0.execute_result(sql, params)?;
+        let rows = self.0.execute_result(&sql.into(), params.into())?;
         Ok(rows)
     }
 
-    pub fn execute_iter(
+    pub fn execute_iter<S: Into<String>, P: Into<Params>>(
         &mut self,
-        sql: &str,
-        params: &[&Value],
+        sql: S,
+        params: P,
     ) -> Result<Vec<AkitaData>, AkitaError> {
-        let rows = self.0.execute_result(sql, params)?;
+        let rows = self.0.execute_result(&sql.into(), params.into())?;
         let datas: Vec<AkitaData> = rows.iter().collect();
         Ok(datas)
     }
 
-    pub fn execute_first(
+    pub fn execute_first<S: Into<String>, P: Into<Params>>(
         &mut self,
-        sql: &str,
-        params: &[&Value],
+        sql: S,
+        params: P,
     ) -> Result<AkitaData, AkitaError> {
         let record: Result<Option<AkitaData>, AkitaError> =
             self.execute_first_opt(sql, params);
@@ -229,10 +232,10 @@ impl AkitaManager {
         }
     }
 
-    pub fn execute_first_opt(
+    pub fn execute_first_opt<S: Into<String>, P: Into<Params>>(
         &mut self,
-        sql: &str,
-        params: &[&Value],
+        sql: S,
+        params: P,
     ) -> Result<Option<AkitaData>, AkitaError> {
         let result: Result<Vec<AkitaData>, AkitaError> = self.execute_iter(sql, params);
         match result {
@@ -249,6 +252,11 @@ impl AkitaManager {
 
 #[allow(unused)]
 impl AkitaEntityManager{
+
+    pub fn new(db: DatabasePlatform) -> Self {
+        AkitaEntityManager(db)
+    }
+
     pub fn start_transaction(&mut self) -> Result<AkitaTransaction, AkitaError> {
         self.0.start_transaction()?;
         Ok(AkitaTransaction {
@@ -260,7 +268,7 @@ impl AkitaEntityManager{
 
     pub fn set_session_user(&mut self, username: &str) -> Result<(), AkitaError> {
         let sql = format!("SET SESSION ROLE '{}'", username);
-        self.0.execute_result(&sql, &[])?;
+        self.0.execute_result(&sql, Params::Nil)?;
         Ok(())
     }
 
@@ -459,7 +467,7 @@ impl AkitaMapper for AkitaEntityManager{
         let where_condition = wrapper.get_sql_segment();
         let where_condition = if where_condition.trim().is_empty() { String::default() } else { format!("WHERE {}",where_condition) };
         let sql = format!("SELECT {} FROM {} {}", &enumerated_columns, &table.complete_name(),where_condition);
-        let rows = self.0.execute_result(&sql, &[])?;
+        let rows = self.0.execute_result(&sql, Params::Nil)?;
         let mut entities = vec![];
         for data in rows.iter() {
             let entity = T::from_data(&data);
@@ -493,7 +501,7 @@ impl AkitaMapper for AkitaEntityManager{
         let where_condition = wrapper.get_sql_segment();
         let where_condition = if where_condition.trim().is_empty() { String::default() } else { format!("WHERE {}",where_condition) };
         let sql = format!("SELECT {} FROM {} {}", &enumerated_columns, &table.complete_name(), where_condition);
-        let rows = self.0.execute_result(&sql, &[])?;
+        let rows = self.0.execute_result(&sql, Params::Nil)?;
         Ok(rows.iter().next().map(|data| T::from_data(&data)))
     }
 
@@ -526,7 +534,7 @@ impl AkitaMapper for AkitaEntityManager{
                 DatabasePlatform::Sqlite(_) => format!("SELECT {} FROM {} WHERE `{}` = ${} limit 1", &enumerated_columns, &table.complete_name(), &field.name, col_len + 1),
                 _ => format!("SELECT {} FROM {} WHERE `{}` = ${} limit 1", &enumerated_columns, &table.complete_name(), &field.name, col_len + 1),
             };
-            let rows = self.0.execute_result(&sql, &[&id.to_value()])?;
+            let rows = self.0.execute_result(&sql, (id.to_value(),).into())?;
             Ok(rows.iter().next().map(|data| T::from_data(&data)))
         } else {
             Err(AkitaError::MissingIdent(format!("Table({}) Missing Ident...", &table.name)))
@@ -562,11 +570,11 @@ impl AkitaMapper for AkitaEntityManager{
         let where_condition = wrapper.get_sql_segment();
         let where_condition = if where_condition.trim().is_empty() { String::default() } else { format!("WHERE {}",where_condition) };
         let count_sql = format!("select count(1) as count from {} {}", &table.complete_name(), where_condition);
-        let count: Count = self.execute_first(&count_sql, &[])?;
+        let count: Count = self.execute_first(&count_sql, ())?;
         let mut page = IPage::new(page, size ,count.count as usize, vec![]);
         if page.total > 0 {
             let sql = format!("SELECT {} FROM {} {} limit {}, {}", &enumerated_columns, &table.complete_name(), where_condition,page.offset(),  page.size);
-            let rows = self.0.execute_result(&sql, &[])?;
+            let rows = self.0.execute_result(&sql, Params::Nil)?;
             let mut entities = vec![];
             for dao in rows.iter() {
                 let entity = T::from_data(&dao);
@@ -597,7 +605,7 @@ impl AkitaMapper for AkitaEntityManager{
             table.complete_name(),
             where_condition
         );
-        let count: Result<Count, AkitaError> = self.execute_first(&sql, &[]);
+        let count: Result<Count, AkitaError> = self.execute_first(&sql, ());
         count.map(|c| c.count as usize)
     }
 
@@ -613,7 +621,7 @@ impl AkitaMapper for AkitaEntityManager{
         let where_condition = wrapper.get_sql_segment();
         let where_condition = if where_condition.trim().is_empty() { String::default() } else { format!("WHERE {}",where_condition) };
         let sql = format!("delete from {} {}", &table.complete_name(), where_condition);
-        let _ = self.0.execute_result(&sql, &[])?;
+        let _ = self.0.execute_result(&sql, Params::Nil)?;
         Ok(())
     }
 
@@ -639,7 +647,7 @@ impl AkitaMapper for AkitaEntityManager{
                 DatabasePlatform::Sqlite(_) => format!("delete from {} where `{}` = ${}", &table.name, &field.name, col_len + 1),
                 _ => format!("delete from {} where `{}` = ${}", &table.name, &field.name, col_len + 1),
             };
-            let _ = self.0.execute_result(&sql, &[&id.to_value()])?;
+            let _ = self.0.execute_result(&sql, (id.to_value(),).into())?;
             Ok(())
         } else {
             Err(AkitaError::MissingIdent(format!("Table({}) Missing Ident...", &table.name)))
@@ -674,10 +682,9 @@ impl AkitaMapper for AkitaEntityManager{
                     None => values.push(Value::Nil),
                 }
             }
-            let v = values.iter().collect::<Vec<_>>();
-            self.0.execute_result(&sql, &v)?;
+            self.0.execute_result(&sql, values.into())?;
         } else {
-            self.0.execute_result(&sql, &[])?;
+            self.0.execute_result(&sql, Params::Nil)?;
         }
         Ok(())
     }
@@ -738,8 +745,7 @@ impl AkitaMapper for AkitaEntityManager{
                     return Err(AkitaError::MissingIdent(format!("Table({}) Missing Ident value...", &table.name)));
                 }
             }
-            let bvalues: Vec<&Value> = values.iter().collect();
-            let _ = self.0.execute_result(&sql, &bvalues)?;
+            let _ = self.0.execute_result(&sql, values.into())?;
             Ok(())
         } else {
             Err(AkitaError::MissingIdent(format!("Table({}) Missing Ident...", &table.name)))
@@ -777,42 +783,40 @@ impl AkitaMapper for AkitaEntityManager{
             }
         }
         let bvalues: Vec<&Value> = values.iter().collect();
-        self.0.execute_result(&sql, &bvalues)?;
+        self.0.execute_result(&sql,values.into())?;
         let rows: Rows = match self.0 {
             #[cfg(feature = "akita-mysql")]
-            DatabasePlatform::Mysql(_) => self.0.execute_result("SELECT LAST_INSERT_ID();", &[])?,
+            DatabasePlatform::Mysql(_) => self.0.execute_result("SELECT LAST_INSERT_ID();", Params::Nil)?,
             #[cfg(feature = "akita-sqlite")]
-            DatabasePlatform::Sqlite(_) => self.0.execute_result("SELECT LAST_INSERT_ROWID();", &[])?,
+            DatabasePlatform::Sqlite(_) => self.0.execute_result("SELECT LAST_INSERT_ROWID();", Params::Nil)?,
         };
         let last_insert_id = rows.iter().next().map(|data| usize::from_data(&data)).unwrap_or_default();
         Ok(last_insert_id)
     }
 
     #[allow(clippy::redundant_closure)]
-    fn execute_result<'a, R>(
+    fn execute_result<'a, R, S: Into<String>, P: Into<Params>>(
         &mut self,
-        sql: &str,
-        params: &[&'a dyn ToValue],
+        sql: S,
+        params: P,
     ) -> Result<Vec<R>, AkitaError>
     where
         R: FromAkita,
     {
-        let values: Vec<Value> = params.iter().map(|p| p.to_value()).collect();
-        let bvalues: Vec<&Value> = values.iter().collect();
-        let rows = self.0.execute_result(sql, &bvalues)?;
+        let rows = self.0.execute_result(&sql.into(), params.into())?;
         Ok(rows.iter().map(|data| R::from_data(&data)).collect::<Vec<R>>())
     }
 
-    fn execute_first<'a, R, S: Into<String>>(
+    fn execute_first<'a, R, S: Into<String>, P: Into<Params>>(
         &mut self,
         sql: S,
-        params: &[&'a dyn ToValue],
+        params: P,
     ) -> Result<R, AkitaError>
     where
         R: FromAkita,
     {
         let sql: String = sql.into();
-        let result: Result<Vec<R>, AkitaError> = self.execute_result(&sql, &params);
+        let result: Result<Vec<R>, AkitaError> = self.execute_result(&sql, params);
         match result {
             Ok(mut result) => match result.len() {
                 0 => Err(AkitaError::DataError("Zero record returned".to_string())),
@@ -823,27 +827,27 @@ impl AkitaMapper for AkitaEntityManager{
         }
     }
 
-    fn execute_drop<'a, S: Into<String>>(
+    fn execute_drop<'a, S: Into<String>, P: Into<Params>>(
         &mut self,
         sql: S,
-        params: &[&'a dyn ToValue],
+        params: P,
     ) -> Result<(), AkitaError>
     {
         let sql: String = sql.into();
-        let _result: Result<Vec<()>, AkitaError> = self.execute_result(&sql, &params);
+        let _result: Result<Vec<()>, AkitaError> = self.execute_result(&sql, params);
         Ok(())
     }
 
-    fn execute_result_opt<'a, R, S: Into<String>>(
+    fn execute_result_opt<'a, R, S: Into<String>, P: Into<Params>>(
         &mut self,
         sql: S,
-        params: &[&'a dyn ToValue],
+        params: P,
     ) -> Result<Option<R>, AkitaError>
     where
         R: FromAkita,
     {
         let sql: String = sql.into();
-        let result: Result<Vec<R>, AkitaError> = self.execute_result(&sql, &params);
+        let result: Result<Vec<R>, AkitaError> = self.execute_result(&sql, params);
         match result {
             Ok(mut result) => match result.len() {
                 0 => Ok(None),
@@ -871,7 +875,19 @@ mod test {
 
     #[test]
     fn get_table_info() {
-        let s = params! { "test" => 1};
+        let s = params! { "test" => 1, "id" => 3, "id"=> 4};
+        let mut sql = "select * from user where id = :id and test = :test and id = :id".to_string();
+        let len = sql.len();
+        let mut values = s.iter().map(|param| {
+            let key = format!(":{}", param.0);
+            let index = sql.find(&key).unwrap_or(len);
+            sql = sql.replace(&key, "?");
+            println!("key: {}, index: {}",key, index);
+            (index, &param.1)
+        }).collect::<Vec<_>>();
+        values.sort_by(|a, b| a.0.cmp(&b.0));
+        let params = values.iter().map(|v| v.1).collect::<Vec<_>>();
+        let params = params.as_slice();
         // let db_url = String::from("mysql://root:password@localhost:3306/akita");
         // let mut pool = Pool::new(AkitaConfig::default()).unwrap();
         // let mut em = pool.entity_manager().expect("must be ok");
@@ -879,8 +895,8 @@ mod test {
         //     .get_table(&TableName::from("public.film"))
         //     .expect("must have a table");
         // println!("table: {:#?}", table);
-        let s = serde_json::to_value("[123,3455,556]").unwrap();
-        println!("{}", s)
+        // let s = serde_json::to_value("[123,3455,556]").unwrap();
+        println!("sql:{}, {:?}", sql, params)
     }
 
     #[test]
