@@ -29,7 +29,6 @@ pub enum Value {
     Timestamp(DateTime<Utc>),
     Interval(Interval),
     SerdeJson(serde_json::Value),
-
     // Point(Point<f64>),
 
     Array(Array),
@@ -206,7 +205,8 @@ where
     }
 }
 
-impl<T> ToValue for &T
+
+impl<'a, T> ToValue for &'a T
 where
     T: ToValue,
 {
@@ -215,6 +215,12 @@ where
     }
 }
 
+impl <'a> From<&'a Value> for Value {
+    fn from(v: &'a Value) -> Value {
+        v.to_owned()
+    }
+}
+
 impl<T> From<T> for Value
 where
     T: ToValue,
@@ -224,6 +230,13 @@ where
     }
 }
 
+// impl<'a, T: ToValue> From<&'a T> for Value {
+//     fn from(x: &'a T) -> Value {
+//         x.to_value()
+//     }
+// }
+
+
 #[derive(Debug)]
 pub enum ConvertError {
     NotSupported(String, String),
@@ -415,4 +428,113 @@ fn parse_naive_date_time(v: &str) -> NaiveDateTime {
             panic!("unable to parse timestamp: {}", v);
         }
     }
-}
\ No newline at end of file
+}
+
+
+#[derive(Debug, Clone, PartialEq)]
+pub enum Params {
+    Nil, // no params
+    Vector(Vec<Value>), // vec
+    Custom(Vec<(String, Value)>), // custom params
+}
+// pub trait ToParam {
+//     fn to_param(&self) -> Params;
+// }
+
+impl From<Vec<Value>> for Params {
+    fn from(x: Vec<Value>) -> Params {
+        Params::Vector(x)
+    }
+}
+
+impl<'a> From<&'a [&'a dyn ToValue]> for Params {
+    fn from(x: &'a [&'a dyn ToValue]) -> Params {
+        let values = x.iter().map(|p| p.to_value()).collect::<Vec<Value>>();
+        Params::Vector(values)
+    }
+}
+
+impl<N, V> From<Vec<(N, V)>> for Params
+where
+    String: From<N>,
+    Value: From<V>,
+{
+    fn from(x: Vec<(N, V)>) -> Params {
+        let mut params = Vec::new();
+        for (name, value) in x.into_iter() {
+            let name = String::from(name);
+            params.push((name, Value::from(value)));
+        }
+        Params::Custom(params)
+    }
+}
+
+impl From<()> for Params {
+    fn from(_: ()) -> Params {
+        Params::Nil
+    }
+}
+
+impl From<Value> for Params {
+    fn from(x: Value) -> Params {
+        Params::Vector(vec![x])
+    }
+}
+
+impl From<String> for Params {
+    fn from(x: String) -> Params {
+        Params::Vector(vec![x.into()])
+    }
+}
+
+impl <'a> From<&'a dyn ToValue> for Params {
+    fn from(x: &'a dyn ToValue) -> Params {
+        Params::Vector(vec![x.to_value().to_owned()])
+    }
+}
+
+macro_rules! into_params_impl {
+    ($([$A:ident,$a:ident]),*) => (
+        impl<$($A: Into<Value>,)*> From<($($A,)*)> for Params {
+            fn from(x: ($($A,)*)) -> Params {
+                let ($($a,)*) = x;
+                let mut params = Vec::new();
+                $(params.push($a.into());)*
+                Params::Vector(params)
+            }
+        }
+    );
+}
+
+impl<'a, T: Into<Params> + Clone> From<&'a T> for Params {
+    fn from(x: &'a T) -> Params {
+        x.clone().into()
+    }
+}
+
+into_params_impl!([A, a]);
+into_params_impl!([A, a], [B, b]);
+into_params_impl!([A, a], [B, b], [C, c]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t], [U, u]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t], [U, u], [V, v]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t], [U, u], [V, v], [W, w]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t], [U, u], [V, v], [W, w], [X, x]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t], [U, u], [V, v], [W, w], [X, x], [Y, y]);
+into_params_impl!([A, a], [B, b], [C, c], [D, d], [E, e], [F, f], [G, g], [H, h], [I, i], [J, j], [K, k], [L, l], [M, m], [N, n], [O, o], [P, p], [Q, q], [R, r], [S, s], [T, t], [U, u], [V, v], [W, w], [X, x], [Y, y], [Z, z]);
